\ignore{
\begin{code}
{-#LANGUAGE BangPatterns #-}
\end{code}
}
\begin{code}
module Pats(placeTile) where

import Data.Bits
import qualified Data.Vector as V
import qualified Data.Vector.Unboxed as U
import qualified Data.Vector.Generic as G
import Tile
\end{code}
\ignore{
\begin{code}
(!)::G.Vector v a=> v a->Int->a
(!)=G.unsafeIndex

(//)::G.Vector v a=>v a->[(Int,a)]->v a
(//)=G.unsafeUpd
\end{code}
}

\begin{lemma}\label{lem:directed}
Given a tileset $\mathtt{tiles}$, $\mathtt{isDirected}\ \mathtt{tiles}=\mathtt{true}$ if and only if
$\mathtt{tiles}$ is a directed DTAS tileset, that is, no two tiles in $\mathtt{tiles}$ have the same input (i.e. south and west) glues.

\begin{proof}

$\mathtt{isDirected}$ works by defining and calling the recursive function
$\mathtt{isDir}$. We prove by induction on {\tt i} that {\tt isDir i s} is
{\tt True} if and only if for all
$j,k\in S_i=\{{\mathtt i},{\mathtt i+1},\ldots |{\tt tiles}|-1\}$,
$(\mathrm{west}_j,\mathrm{south}_j)\not\in{\mathtt s}$,
and $(\mathrm{west}_j,\mathrm{south}_j)\neq(\mathrm{west}_k,\mathrm{south}_k)$.

\hfill
\begin{code}

staysDirected::Int->Int->Int->Int->U.Vector Tile->Bool
staysDirected a0_ b0_ a1_ b1_ tiles=
  let (!a0,!b0)=if a0_<b0_ then (a0_,b0_) else (b0_,a0_)
      (!a1,!b1)=if a1_<b1_ then (a1_,b1_) else (b1_,a1_)
      isDir i j=
        if i>=U.length tiles then True else
          if j>=i || (south (tiles!i)/=a0 && south (tiles!i)/=b0)
              || (west (tiles!i)/=a1 && west (tiles!i)/=b1)
          then
            isDir (i+1) 0
          else
            if (south (tiles!j)==a0 || south (tiles!j)==b0)
               && (west (tiles!j) == a1 || west (tiles!j)==b1) then
              False
            else
              isDir i (j+1)
  in
   isDir 0 0
{-
isDirected::U.Vector Tile->Bool
isDirected !tiles=
  let isDir i j=
        if i>=U.length tiles then True else
          if j>=i then isDir (i+1) 0 else
            if south (tiles!i) == south (tiles!j)
               && west (tiles!i) == west (tiles!j) then
              False
            else
              isDir i (j+1)
  in
   isDir 0 0
-}
\end{code}
\end{proof}
\end{lemma}

We now proceed to the proof of the {\tt merge} function.
Its formal specification of this function is given by the following Lemma:

\begin{lemma}\label{lem:merge}
Given a tileset $\mathtt{ts}$, a color $\mathtt{c}$, an index into the tileset $\mathtt{i}$, a pair of north/south glues $\mathtt{a0\_}$ and $\mathtt{b0\_}$, and a pair of east/west glues $\mathtt{a1\_}$ and $\mathtt{b1\_}$, the function $\mathtt{merge}$ returns a tileset in which the $\mathtt{i}$th tile of $\mathtt{ts}$ is set to color $\mathtt{c}$, and all north/south glues in $\mathtt{ts}$ equal to $\mathtt{max(a0\_,b0\_)}$ are set to $\mathtt{min(a0\_,b0\_)}$ if this value is nonnegative (and left unchanged else), and all east/west glues in $\mathtt{ts}$ equal to $\mathtt{max(a1\_,b1\_)}$ are set to $\mathtt{min(a1\_,b1\_)}$ if this value is nonnegative (and left unchanged else).
\end{lemma}

\begin{proof}

In $\mathtt{merge}$, the pair $\mathtt{(a0,b0)}$ is formed such that
$\mathtt{a0} = \mathtt{min(a0\_,b0\_)}$ and $\mathtt{b0} =
\mathtt{max(a0\_,b0\_)}$.  Similarly, the pair $\mathtt{(a1,b1)}$ is formed such
that $\mathtt{a1} = \mathtt{min(a1\_,b1\_)}$ and $\mathtt{b1} =
\mathtt{max(a1\_,b1\_)}$.

The rest of the function does exactly our claim: a new tileset vector is
generated, in which new tiles are created, that correspond to our claim.

\hfill

\begin{code}
merge::Int->Int->Int->Int->Int->Int->U.Vector Tile->U.Vector Tile
merge a0_ b0_ a1_ b1_ i0 c ts=
  let (!a0,!b0)=if a0_<b0_ then (a0_,b0_) else (b0_,a0_)
      (!a1,!b1)=if a1_<b1_ then (a1_,b1_) else (b1_,a1_)
  in
   U.generate (U.length ts)
     (\i->
       let !u=if i==i0 then (ts ! i)`withC`c else ts ! i
           !v=if south u==b0 && a0>=0 then u`withS`a0 else u
           !w=if north v==b0 && a0>=0 then v`withN`a0 else v
           !x=if west w==b1 && a1>=0 then w`withW`a1 else w
       in
        if east x==b1 && a1>=0 then x`withE`a1 else x
     )
\end{code}

\end{proof}


Finally, the core of our algorithm is the following recursive function, $\mathtt{placeTile}$, which moves through all locations in the pattern in the ordering shown by Figure~\ref{fig:order} and places tiles from the current tile set (often modifying the tile set, too) as long as it is able to.  By making recursive calls which attempt all possibilities, it ensures that the full set of possible tile sets (up to isomorphism) is explored and returns exactly those which self-assemble the given pattern. The arguments to $\mathtt{placeTile}$ are:

\begin{enumerate}
    \item[] {\tt share}: a boolean value set by the server, possibly telling the client to ``share'' the current job
    \item[] {\tt save}: a function that we have explained in Lemma \ref{lem:dowork}, that {\tt placeTile} can use to ``save'' intermediate results in case it is asked to reshare, or killed (for instance if it runs on a cluster).
    \item[] {\tt results}: a list of results that have been found so far.
    \item[] {\tt jobs}: a list of jobs to treat. Each job contains four relevant fields for the actual computation:
\begin{enumerate}
    \item[] {\tt posX},{\tt posY}: the coordinates of the current position in the assembly which $\mathtt{placeTile}$ should attempt to fill with a tile

    \item[] {\tt tileset}: the current tileset (which is a vector of integer values which are each the concatenated integer values representing the properties of a tile type)

    \item[] {\tt assembly}: the current assembly (which is a two-dimensional vector storing the index of the tile type, in the tileset, which is located at each pair of $(x,y)$ coordinates
\end{enumerate}
\end{enumerate}

Note that $\mathtt{placeTile}$ also makes use of the globally defined
two-dimensional vector $\mathtt{pattern}$ which, at each location representing a
pair of $(x,y)$ coordinates, defines one of two colors (i.e. 0 (black) or 1
(white)) for the pattern at that location.

\begin{lemma}\label{lem:placeTile}

For any list of jobs $\mathtt{j}_{\mathtt 0}$, any value of {\tt share} and any function {\tt
save}, {\tt placeTile share save [] $\mathtt{j}_{\mathtt 0}$} returns the list of all subjobs of jobs of
$\mathtt{j_0}$ that have not been explored, and all results that have been found during
the exploration of the explored subjobs of $\mathtt{j}_{\mathtt 0}$.

Moreover, all its calls to {\tt save} are of the form {\tt save j r}, where
{\tt j} is the list of all subjobs of $\mathtt{j}_{\mathtt 0}$ that have not been completely
explored, and $\mathtt{r}$ is the list of all results that have been found in the
exploration of all other subjobs of $\mathtt{j}_{\mathtt 0}$.

\end{lemma}

%Beginning from assembly $\mathtt{assemb}$, tileset $\mathtt{tiles}$, and location $(x,y)$, if $\mathtt{share} = \mathtt{false}$ the function $\mathtt{placeTiles}$ returns a list of all possible pairs of completed assemblies and the tilesets of size $\leq 13$ which can self-assemble them.


%If $\mathtt{share} = \mathtt{true}$, $\mathtt{placeTile}$ creates a list of ``jobs'' which continue from the next tile location and sends them to the server, which will ensure that they will be completed by some client.  The server and the mechanism for distributing jobs is described in Sections \ref{subsect:server} and \ref{subsect:client}.

\begin{proof}

We will prove, by induction on the number of subjobs of
$\mathtt{j}_{\mathtt{0}}$, that for all values of $\mathtt{r}$ and {\tt j},
{\tt placeTile share save r j} is the concatenation of {\tt r} with all the results
found in the exploration, and all the subjobs of jobs of {\tt j} that have not
been explored.

Moreover, we will prove the following invariant:

\hfill

\begin{invariant}
\label{l}

The recursive calls of {\tt placeTile} are all such that the {\tt results} and
{\tt jobs} arguments verify the condition that {\tt jobs} is the list of all
subjobs of the initial job list that have not been explored and contains no
results, and {\tt results} is the list of all results that have been found
during the exploration of all other subjobs of the initial job list.

\end{invariant}

The first case is when the list of jobs to explore is empty. In this case,
we simply return the list of found results, and the claim holds.

\hfill
\begin{code}
placeTile::Bool -> ([Job] -> [Job] -> IO ()) -> [Job] -> [Job] -> IO [Job]
placeTile _ _ res []=return res
\end{code}
\hfill

Now, even though in our protocol, no results are ever sent to the {\tt
placeTile} function in the job list argument, we need to treat this case for our
claim to hold.

\hfill
\begin{code}
placeTile !share save results (j@R{}:js)=
  placeTile share save (j:results) js
\end{code}
\hfill

We now proceed to the proof of the main case.

\hfill
\begin{code}
placeTile !share save results (j@J{}:js)=do {
\end{code}
The following line is the only place where {\tt placeTile} calls {\tt save}.
By invariant \ref{l} on the recursive calls of {\tt placeTile}, our claim
on the calls to {\tt save} clearly holds.
\begin{code}
    save (j:js) results;
    let { !x=posX j;
          !y=posY j;
          !tiles=tileset j;
          !assemb=assembly j;
\end{code}

First, the variable $\mathtt{inS}$ is the glue to the south of the location
$(x,y)$ to be tiled (i.e. its south input).  If the location to the south is
outside of the pattern or if there is no tile there, then the glue value of $-1$
is used.  In an analogous manner, the variable $\mathtt{inW}$ is the value of
the input glue to the west.

\begin{code}
          !inS=
            if y>=1 then
              let !a=(assemb!(y-1)!x) in
              if a>=0 then
                north $! tiles!a
              else
                -1
            else
              -1;
          !inW=
            if x>=1 then
              let !a=(assemb!y!(x-1))in
              if a>=0 then
                east $! tiles!a
              else
                -1
            else
              -1
        };
\end{code}

If {\tt inS} (respectively {\tt inW}) is negative, and we are not on the south
(respectively west) border, then there is no more tile we can add on the current
row (respectively column). Therefore, we must start a new column (respectively a
new row). Remark that since we keep alternating between adding rows and columns,
we maintain the invariant that $\mathtt{posX}\geq\mathtt{posY}$ exactly when we
are adding a new row.
This is what the following code does. Invariant \ref{l}, on {\tt placeTile}'s recursive
calls, is clearly preserved by all the calls in this portion of the code.

\begin{code}
    if inS<0 && y>0 then
      if x<U.length (pattern ! 0) then
        placeTile share save results (j { posX=x,posY=0 }:js)
      else
        if y+1>=V.length (pattern) then
          placeTile share save (R { tileset=tiles,assembly=assemb }:results) js
        else
          placeTile share save results (j { posX=0,posY=y+1 }:js)
    else
      if inW<0 && x>0 then
        if y<V.length (pattern) then
          placeTile share save results (j { posX=0,posY=y }:js)
        else
          if x+1>=U.length (pattern ! 0) then
            placeTile share save (R { tileset=tiles,assembly=assemb }:results) js
          else
            placeTile share save results (j { posX=x+1,posY=0 }:js)
      else
\end{code}

Else, both the south and west glues are defined, or we are at the beginning of a
row or a column. Hence, there are two possible cases: either there is already a
tile with matching south and west glues, or there is none.  In the first case,
we have no choice but to place that tile at the current position, and move on to
the next position, which is done in case {\tt Just p}:

\begin{code}
        let { (!nextX,!nextY)=if y>x then (x+1,y) else (x,y+1);
              col=pattern!y ! x;
              possible=U.findIndex (\a->south a==inS && west a==inW) tiles }
        in
         case possible of {
           Just p->
              let { !color_h=color (tiles ! p) } in
              if color_h==col || color_h==mgl then
                placeTile share save results
                (j { posX=nextX,posY=nextY,
                     tileset=
                       if color_h==mgl then tiles // [(p,(tiles !p)`withC`col)]
                       else tiles,
                     assembly=assemb // [(y,(assemb!y) // [(x,p)])]
                   }:js)
              else
                placeTile share save results js;

\end{code}

Or there is no matching tile, in which case we simply try all tiles that can be placed
at the current position, which fall in either of two cases:

\begin{itemize}

\item tiles whose color matches the pattern's color at the current position.
\item tiles that have not yet been used, i.e. whose color is not yet defined.
We only need to consider one of them, because we do not consider solutions that are
equivalent by renaming. Therefore, we use the {\tt seenBlank} argument of {\tt
tryAllTiles} to keep track of whether we have already tried an uncolored tile.

\end{itemize}

In both cases, we merge these tiles' west and south glues with {\tt inW} and
{\tt inS}, respectively: according to Lemma \ref{lem:merge}, this means that we
adjust the tileset so that the chosen tile can be placed without mismatches at
the current position.

Formally, we can easily prove by induction on {\tt i}, that {\tt tryAllTiles i
False l} (respectively {\tt tryAllTiles i True l}) is the list of all subjobs of
{\tt j}, that try to place a tile of index {\tt i} or larger in the tileset, not
including (respectively including) uncolored tiles, along with the jobs of list
{\tt l}.

\begin{code}
              Nothing->
                let { tryAllTiles i seenBlanks list=
                         if i>=U.length tiles then
                           list
                         else
                           let !t=tiles !i in
                           if color t==col || (color t==mgl && not seenBlanks) then
                              if staysDirected inS (south t) inW (west t) tiles then
                                let !tiles'=
                                      merge inS (south t) inW (west t) i col tiles
                                    !next=
                                      J { posX=nextX,posY=nextY,
                                          tileset=tiles',
                                          assembly=
                                            (assemb // [(y,(assemb!y) // [(x,i)])]),
                                          k=0 }
                                in
                                 tryAllTiles (i+1)
                                 (seenBlanks || (color t==mgl))
                                 (next:list)
                              else
                                tryAllTiles (i+1) seenBlanks list
                           else
                             tryAllTiles (i+1) seenBlanks list;
                      nextJobs=tryAllTiles 0 False js }
                in
\end{code}
First, if we were asked to share, return all newly created jobs, along with
{\tt js}, the remaining jobs after {\tt j} is divided into subjobs. In this
case, the claim holds: {\tt placeTile} does return all jobs and results found
during the exploration.
\begin{code}
                 if share then return (results++nextJobs)
\end{code}

From our proof of the {\tt tryAllTiles} function, the following recursive
call to {\tt placeTile} preserves the invariant on {\tt placeTile}'s recursive
calls. Indeed, {\tt results} has not changed, and {\tt nextJobs} now contains
{\tt js}, along with all subjobs of {\tt j} (up to renaming of unused tiles).
By the definitions of jobs and subtasks (see Definitions \ref{def:tasks} and
\ref{def:jobs}), invariant \ref{l} is clearly preserved.

\begin{code}
                 else placeTile share save results nextJobs
           }
    }
\end{code}
\end{proof}
