\begin{code}
{-#LANGUAGE FlexibleInstances,MultiParamTypeClasses #-}
{-#OPTIONS -fno-warn-orphans #-}

import Parry.Server
import Parry.WebUI
import Tile
import Keys
import qualified Data.Vector.Unboxed as U
import qualified Data.Vector as V
import Control.Concurrent
import System.Posix
import System.Exit
import qualified Data.ByteString.Char8 as B
import qualified Data.ByteString.Lazy as LB
import Data.ByteString.Lazy.Builder
import Data.Binary
import Data.Monoid

instance Exhaustive Job where
  depth j=max (posX j) (posY j)
  killed j=k j
  kill j=j { k=k j+1 }

instance Result Job [(String,Job)] where
  addResult h r j=(h,j):r



initialTileset::U.Vector Tile
initialTileset=
  U.generate 13 (\i->((0)`withS`(2*i)`withN`(2*i+1)
                      `withW`(2*i)`withE`(2*i+1)`withC`mgl))

initial::Job
initial=J { tileset=initialTileset,
            assembly=V.map (U.map (\_->(-1))) pattern,
            posX=0,
            posY=0,
            k=0 }




closeFid::Fd->IO ()
closeFid f=do {
  devnull<-openFd "/dev/null" ReadWrite Nothing defaultFileFlags;
  closeFd f;
  _<-dupTo devnull f;
  return ()
  }


daemonize :: IO () -> IO ()
daemonize program = do{
  let {p=do{
          _<-createSession;
          _<-forkProcess p';
          exitImmediately ExitSuccess
          };
       p'=do{
         closeFid stdInput;
         closeFid stdOutput;
         closeFid stdError;
         _<-installHandler sigHUP Ignore Nothing;
         program
         }};
  _<-forkProcess p;
  exitImmediately ExitSuccess
  }


instance Html Job where
  toHtml j=
    byteString $
    B.pack $ "<span style=\"font-family:monospace\">"++
    (case j of {
        R {}->"R ("++show (U.toList $ tileset j,map U.toList $ V.toList $ assembly j)++")";
        J {}->"J ("++show (posX j,posY j,U.toList $ tileset j)++")"
        })++"</span>"


instance Html [(String,Job)] where
  toHtml l=
    (byteString $ B.pack "<p style=\"font-family:monospace\">")
    `mappend`
    (mconcat $
     map (\(x,y)->mconcat [byteString $ B.pack "(",
                           byteString $ B.pack x,
                           byteString $ B.pack ", ",
                           toHtml y,
                           byteString $ B.pack ")"]) l)
    `mappend`
    (byteString $ B.pack "</p>")



main::IO ()
main=daemonize $ do
  state<-stateFromFile "state" [initial] ([]::[(String,Job)])
  modifyMVar_ state $ \st->
    return $ st { authorizedKeys=public:(authorizedKeys st) }
  _<-forkIO $ saveThread "state" 100000000 state
  _<-forkIO $ webUI 8000 state
  server defaultConfig state

\end{code}
