-- | A tool to analyze states from the server, to get more detailed statistics
-- than what the web UI gives.

{-#LANGUAGE FlexibleInstances,MultiParamTypeClasses #-}
import Parry.Server
import Tile
import Data.Binary
import qualified Data.ByteString.Lazy as B
import qualified Data.Vector.Unboxed as U
import qualified Data.Vector as V
import Control.Concurrent
import qualified Data.Set as S
import qualified Data.Map as M

instance Exhaustive Job where
  depth j=max (posX j) (posY j)
  killed j=k j
  kill j=j { k=k j+1 }

instance Result Job [(String,Job)] where
  addResult h r j=(h,j):r


initialTileset::U.Vector Tile
initialTileset=
  U.generate 13 (\i->((0)`withS`(2*i)`withN`(2*i+1)
                      `withW`(2*i)`withE`(2*i+1)`withC`mgl))

initial::Job
initial=J { tileset=initialTileset,
            assembly=V.map (U.map (\_->(-1))) pattern,
            posX=0,
            posY=0,
            k=0 }

placed j=
  let m=max (posX j) (posY j) in
  m*m+(if posX j>posY j then posY j-1 else
         if posY j>posX j then posY j+posX j-1 else
           posX j+posY j
      )

main=do
  state<-stateFromFile "state" [initial] ([]::[(String,Job)])
  st<-withMVar state return
  let m=S.foldl' (\m (d,x)->M.insertWith S.union (placed x) (S.singleton x) m) M.empty $ jobs st
  print $ M.size $ ongoing st
  mapM_ print $ M.toList $ M.map S.size m
  --mapM_ print $ M.toList $ ongoing st
